//===------------------------- cxa_unexpected.cpp -------------------------===//
//
// Part of the LLVM Project, under the Apache License v2.0 with LLVM Exceptions.
// See https://llvm.org/LICENSE.txt for license information.
// SPDX-License-Identifier: Apache-2.0 WITH LLVM-exception
//
//===----------------------------------------------------------------------===//

#include <exception>
#include "cxxabi.h"
#include "cxa_exception.hpp"

namespace __cxxabiv1
{

extern "C"
{

}

}  // namespace __cxxabiv1

